#-----------------------------------------------------------------------------
# Copyright (c) 2013-2019, PyInstaller Development Team.
#
# Distributed under the terms of the GNU General Public License with exception
# for distributing bootloader.
#
# The full license is in the file COPYING.txt, distributed with this software.
#-----------------------------------------------------------------------------

# Python wrapper for LibRaw (https://pypi.python.org/pypi/rawpy)
# Tested with version 0.3.5

hiddenimports = ['numpy', 'enum']
